from .base import BaseNotifier, Encoding
from .discord import DiscordNotifier
from .email import EmailNotifier
from .slack import SlackNotifier
from .telegram import TelegramNotifier


class Notifier(BaseNotifier):
    """
    A notifier factory which chooses the notifier based on the "type" param.
    """

    NOTIFIERS = [DiscordNotifier, EmailNotifier, SlackNotifier, TelegramNotifier]

    def __new__(cls, type: str, *args, **kwargs):  # pragma: no cover - no complexity
        for notifier in cls.NOTIFIERS:
            if type == notifier.TYPE:  # type: ignore
                return notifier(*args, **kwargs)
        raise ValueError(f"Unknown notifier type: {type}")



from enum import Enum
from typing import Literal


__all__ = ["BaseNotifier", "Encoding"]


class Encoding(Enum):
    ASCII = 1
    UTF8 = 2


class BaseNotifier:  # pragma: no cover - base-class for typing purposes
    """A base-class for notifiers to allow typing."""

    TYPE: str = "UNKNOWN"
    ENCODING: Encoding = Encoding.UTF8

    def notify(self, text: str) -> None:
        """Sends a message."""
        raise NotImplementedError()



import logging

from .base import BaseNotifier


__all__ = ["SlackNotifier"]


class SlackNotifier(BaseNotifier):
    """A Slack notifier."""

    TYPE = "slack"

    def __init__(self, token: str, channel_id: str):
        try:
            from slack_sdk import WebClient
        except ModuleNotFoundError:  # pragma: no cover - no complexity here
            logging.error("Please install the `slack_sdk` package.")
            exit(1)

        self._client = WebClient(token=token)
        self._channel_id = channel_id

    def notify(self, text: str) -> None:
        """Sends a message."""
        self._client.chat_postMessage(channel=self._channel_id, text=text)



import asyncio
import logging

from .base import BaseNotifier


__all__ = ["DiscordNotifier"]


class DiscordNotifier(BaseNotifier):
    """A Discord notifier."""

    TYPE = "discord"

    def __init__(self, token: str, channel_id: int):
        self._token = token
        self._channel_id = channel_id

    async def _async_notify(self, text: str) -> None:
        """Sends a message asynchronous."""
        try:
            from hikari import RESTApp
        except ModuleNotFoundError:  # pragma: no cover - no complexity here
            logging.error("Please install the `hikari` package.")
            exit(1)

        rest_app = RESTApp()
        async with rest_app.acquire(self._token, "Bot") as client:
            channel = await client.fetch_channel(self._channel_id)
            await client.create_message(channel, text)  # type: ignore

    def notify(self, text: str) -> None:
        """Sends a message."""
        asyncio.run(self._async_notify(text))



import logging
import os
import yaml

from celery.schedules import crontab


__all__ = ["BASE_DIR", "CONFIG_FILE", "BROKER_DIR", "CONFIG", "DB_FILENAME"]

BASE_DIR = os.getenv("FIKKIE_BASE_DIR", os.path.expanduser("~/.fikkie"))
CONFIG_FILE = os.getenv("FIKKIE_CONFIG", os.path.join(BASE_DIR, "config.yaml"))
BROKER_DIR = os.getenv("FIKKIE_BROKER_DIR", os.path.join(BASE_DIR, "broker"))
DB_FILENAME = os.getenv("FIKKIE_DB_FILENAME", os.path.join(BASE_DIR, "db.json"))
LOG_FILE = os.getenv("FIKKIE_LOG_FILE", os.path.join(BASE_DIR, "fikkie.log"))
PID_FILE = os.getenv("FIKKIE_PID_FILE", os.path.join(BASE_DIR, "fikkie.pid"))


def load_config() -> dict:  # pragma: no cover - no complexity here
    """Parse the config from a given file."""
    try:
        with open(CONFIG_FILE, "r") as f:
            return yaml.safe_load(f) or {}
    except FileNotFoundError as e:
        raise FileNotFoundError("Could not find config file, please run `fikkie init`.")
    except yaml.YAMLError as e:
        logging.error(f"Could not parse config: {e}")
        exit(1)


def get_schedule() -> tuple:
    """Returns the Celery schedule and timezone."""
    try:
        config = load_config()
    except FileNotFoundError:  # pragma: no cover - no complexity here
        config = {}

    heartbeat_config = config.get("heartbeat", {})

    timezone = heartbeat_config.get("timezone", "UTC")
    schedule = {"tick": {"task": "fikkie.main.tick", "schedule": 60}}

    if heartbeat_config.get("enable", True):
        heartbeat_schedule_config = heartbeat_config.get("schedule", {})
        heartbeat_schedule = {
            "minute": heartbeat_schedule_config.get("minute", 0),
            "hour": heartbeat_schedule_config.get("hour", 12),
            "day_of_week": heartbeat_schedule_config.get("day_of_week", "*"),
            "day_of_month": heartbeat_schedule_config.get("day_of_month", "*"),
        }
        schedule["heartbeat"] = {
            "task": "fikkie.main.heartbeat",
            "schedule": crontab(**heartbeat_schedule),
        }

    return schedule, timezone



from setuptools import setup

with open("README.md", "r") as f:
    long_description = f.read()

setup(
    name="fikkie",
    version="0.5.3",
    description=(
        "The simplest tool for lightweight monitoring over SSH, compatible with all "
        "your favorite messengers."
    ),
    license="MIT",
    long_description=long_description,
    author="Joris Hartog (@nootr)",
    long_description_content_type="text/markdown",
    url="https://github.com/nootr/fikkie/",
    packages=["fikkie", "fikkie.notifiers"],
    scripts=["scripts/fikkie"],
    install_requires=[
        "celery==5.3.6",
        "PyYAML==6.0",
        "tinydb==4.8.0",
        "typing_extensions==4.0.1",
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Framework :: Celery",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: MacOS",
        "Operating System :: POSIX",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Topic :: System :: Monitoring",
    ],
)
